import os
import requests
import warnings

GITHUB_API_URL = 'https://api.github.com'

def file_writer(folder, filename, content):
    with open(os.path.join(folder, filename), 'w') as f:
        f.write(content)

def folder_creator(base_dir, folder_name):
    new_folder = os.path.join(base_dir, folder_name)
    if not os.path.exists(new_folder):
        os.mkdir(new_folder)
    
    return new_folder


def check_user(github_username):
    """Checks how many public repos user have"""
    users_url = '/'.join([GITHUB_API_URL, 'users', github_username])
    req = requests.get(users_url)
    if req.status_code == 200:
        user_public_repo = req.json()['public_repos']
        # How many pages to crawl through 100 per page
        return int(user_public_repo / 100) + 1
    else:
        return Exception(req.text)


def request_repos(*args):
    page, github_username, package_name, url = args
    req = requests.get(url, params={
        'per_page': 100,
        'page': page
    })
    if req.status_code == 200:
        filtered_repo = list(filter(lambda x: x['full_name'] == f'{github_username}/{package_name}', req.json()))
        if len(filtered_repo) > 0:
            return True
        return False
    else:
        return Exception(req.text)


def check_repos(max_page, *args):
    start_page = 0
    while start_page < max_page:
        start_page += 1
        if request_repos(start_page, *args):
            yield True


def check_package(package_name, github_username):
    url = '/'.join([GITHUB_API_URL, 'users', github_username, 'repos'])
    pages = check_user(github_username)
    rep = check_repos(pages, github_username, package_name, url)
    for r in rep:
        if r:
            warnings.warn(f'{package_name} exists on {github_username} Github account!')
            return r

def create_repo(package_name, package_description, github_auth):
    try:
        github_username = github_auth['auth'].username
        package_exists = check_package(package_name, github_username)
        if not package_exists:
            payload = {
                'name': package_name,
                'description': package_description,
                'homepage': '/'.join(['https://github.com', github_username, package_name]),
                'private': False,
                'has_issues': True,
                'has_projects': True,
                'has_wiki': True,
            }
            post_url = '/'.join([GITHUB_API_URL, 'user', 'repos'])
            req = requests.post(post_url, json=payload, auth=github_auth['auth'], headers=github_auth['headers'])
            if req.status_code in [201, 200]:
                return req.json()['clone_url']
            else:
                return Exception(req.text)
    except Exception as e:
        print(e)



import os
import shutil
import sys
import re
import datetime
import subprocess
import argparse

import requests
from munch import Munch

import yaml

from doctr.local import GitHub_login

from .utils import (folder_creator, file_writer, create_repo)
from .templates import (tests_example, 
                        readme_template, 
                        dev_dependencies, 
                        setup_template, 
                        setup_cfg_template, 
                        manifest, 
                        travis_template, 
                        flake8_template, 
                        contributing_md, 
                        authors_md)

__author__ = 'Landung Setiawan'
from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

BASE_DIR = os.path.abspath(os.path.curdir)
LICENSES = requests.get('https://api.github.com/licenses').json()

def create_package_dir(package_name):
    return folder_creator(BASE_DIR, package_name)

def create_tests(package_dir):
    tests_dir = folder_creator(package_dir, 'tests')
    file_writer(tests_dir, '__init__.py', '')
    file_writer(tests_dir, 'test_example.py', tests_example())

def create_travis(package_dir, package_name):
    file_writer(package_dir, '.travis.yml', travis_template(package_name))

def create_flake8(package_dir, package_name):
    file_writer(package_dir, '.flake8', flake8_template(package_name))

def create_authors(package_dir, author_name, author_email):
    file_writer(package_dir, 'AUTHORS.md', authors_md(author_name, author_email))

def create_contributing(package_dir, package_name, github_username):
    file_writer(package_dir, 'CONTRIBUTING.md', contributing_md(package_name, github_username))

def create_notebooks_folder(package_dir):
    notebooks_dir = folder_creator(package_dir, 'notebooks')
    file_writer(notebooks_dir, '.gitkeep', '')

def create_license(selection, package_dir, author_name):
    lic = LICENSES[selection]
    license_detail = requests.get(lic['url']).json()
    license_text = license_detail['body']
    year = datetime.datetime.now().year

    license_text = license_text.replace('[year]', f'{year}')
    license_text = license_text.replace('[fullname]', f'{author_name}')

    file_writer(package_dir, 'LICENSE', license_text)

    return license_detail

def create_setup(package_dir, package_name, package_description, 
                author_name, author_email, license_detail):
    file_writer(package_dir, 
                'setup.py', 
                setup_template(
                    package_name, 
                    package_description, 
                    author_name, 
                    author_email, 
                    license_detail)
                )
    file_writer(package_dir, 'setup.cfg', setup_cfg_template(package_name))

def create_readme(package_dir, package_name, package_description):
    readme_text = readme_template(package_name, package_description)
    file_writer(package_dir, 'README.md', readme_text)

def create_requirements(package_dir, dependencies, **kwargs):
    cleaned_dependencies = list(map(lambda d: d.strip(' '), dependencies.split(',')))
    file_writer(package_dir, 'requirements.txt', '\n'.join(cleaned_dependencies))
    additional_text = kwargs.get('additional_text', '')
    file_writer(package_dir, 'requirements-dev.txt', dev_dependencies(additional_text))

def create_manifest(package_dir, **kwargs):
    additional_text = kwargs.get('additional_text', '') 
    file_writer(package_dir, 'MANIFEST.in', manifest(additional_text))

def use_versioneer():
    cmd = ['versioneer', 'install']
    subprocess.run(cmd)

def create_sphinx_docs(package_dir, package_name, 
                       package_description, author_name, 
                       **kwargs):
    docs_dir = folder_creator(package_dir, 'docs')
    cmd = ['sphinx-quickstart', '--sep', 
           f'--project={package_name}', f'--author="{author_name}"', 
           '--ext-autodoc', '--ext-viewcode', 
           '--extensions=sphinx.ext.napoleon', '--extensions=nbsphinx', 
           '--makefile', '--dot=_', 
           '--release=""', '-v', '""', 
           '--suffix=.rst', '--language=en', 
           '--master=index', '-q', '--no-batchfile', f'{docs_dir}']
    subprocess.run(cmd)

def init_package_code_dir(package_dir, package_name, **kwargs):
    package_code_dir = folder_creator(package_dir, package_name)

    author_name = kwargs.get('author_name', '')
    init_text = f"""__author__ = '{author_name}'"""
    file_writer(package_code_dir, '__init__.py', init_text)

def init_git(package_dir):
    os.chdir(package_dir)
    git = subprocess.run(['git', 'init'])

    gitignore = requests.get('https://raw.githubusercontent.com/github/gitignore/master/Python.gitignore').text
    gitignore_text = f"""{gitignore}\n# ipynb\n.ipynb_checkpoints\n"""
    file_writer(package_dir, '.gitignore', gitignore_text)

def get_user_input(info_file=None):
    args = dict()
    if info_file:
        ymldct = yaml.load(info_file)
        args['package_name'] = ymldct['name']
        args['package_description'] = ymldct['description']
        args['author_name'] = ymldct['author']['name']
        args['author_email'] = ymldct['author']['email']
        args['dependencies'] = ','.join(ymldct['dependencies'])
        args['gh_username'] = ymldct.get('github-id', 'someuser')
    else:
        args['package_name'] = input('Enter package name: ')
        args['package_description'] = input('Enter initial package description: ')
        args['author_name'] = input('Primary Author Name: ')
        args['author_email'] = input('Primary Author Email: ')
        args['dependencies'] = input('Package dependencies (comma separated): ')
        args['gh_username'] = input('What is your github username: ')

    license_list = list(map(lambda x: f"{x[0]}: {x[1]['name']}", enumerate(LICENSES)))
    license_list_str = '\n'.join(license_list)
    args['license'] = int(input(f"{license_list_str}\nSelect number: "))
    
    return Munch(**args)

def init(info_file=None, init_github=False):
    # Get user inputs
    inputs = get_user_input(info_file)
    git_url = None
    if init_github:
        # Get github login
        gh_auth = GitHub_login()
        github_username = gh_auth['auth'].username
        # Initialize repo
        git_url = create_repo(inputs.package_name, inputs.package_description, gh_auth)
    else:
        github_username = inputs.gh_username
    # Create package directory
    package_dir = create_package_dir(inputs.package_name)
    # Initialize git repo
    init_git(package_dir)
    # Create package code directory
    init_package_code_dir(package_dir, inputs.package_name, author_name=inputs.author_name)
    # Create notebooks directory
    create_notebooks_folder(package_dir)
    # Create license
    license_detail = create_license(inputs.license, package_dir, inputs.author_name)
    # Create flake8
    create_flake8(package_dir, inputs.package_name)
    # Create authors
    create_authors(package_dir, inputs.author_name, inputs.author_email)
    # Create contributing
    create_contributing(package_dir, inputs.package_name, github_username)
    # Create readme
    create_readme(package_dir, inputs.package_name, inputs.package_description)
    # Create requirements
    create_requirements(package_dir, inputs.dependencies)
    # Create setup.py
    create_setup(package_dir, 
                inputs.package_name, 
                inputs.package_description, 
                inputs.author_name, 
                inputs.author_email, 
                license_detail)
    # Create Manifest.in
    create_manifest(package_dir)
    # Create tests
    create_tests(package_dir)
    # Create travis
    create_travis(package_dir, inputs.package_name)
    # Use Versioneer
    use_versioneer()
    # Create initial sphinx docs
    create_sphinx_docs(package_dir, 
                       inputs.package_name, 
                       inputs.package_description, 
                       inputs.author_name)
    # Commit changes
    subprocess.run(['git', 'add', '.'])
    subprocess.run(['git', 'commit', '-m', 'Initialize package repository'])
    # Push to github
    if git_url:
        subprocess.run(['git', 'remote', 'add', 'origin', git_url])
        subprocess.run(['git', 'push', '-u', 'origin', 'master'])













import argparse

from . import init

def parse_args():
    parser = argparse.ArgumentParser(description='Package Creator')
    subparsers = parser.add_subparsers(dest='cmd', help='Additional Commands')

    init_parser = subparsers.add_parser('init', help='Initialize a repository')
    init_parser.add_argument('-f','--file', type=argparse.FileType('r'), help='Info file')
    init_parser.add_argument('-gh', '--github', action='store_true', help='Upload to github')

    return parser.parse_args()

def main():
    args = parse_args()
    if args.cmd == 'init':
        init(info_file=args.file, init_github=args.github)

if __name__ == '__main__':
    main()



def readme_template(package_name, package_description):
    return f"""# {package_name}\n
{package_description}\n
This package has been automatically generated, 
please edit this file for better description."""

def dev_dependencies(additional_text):
    return f"""# Testing packages
pytest

# Linting packages
flake8
flake8-builtins
flake8-comprehensions
flake8-import-order
flake8-mutable
flake8-print
flake8-quotes

# Debugging packages
ipython
ipykernel

# Documentation
sphinx
doctr
nbsphinx

{additional_text}
"""

def setup_template(package_name, package_description, author_name, author_email, license_detail):
    return f"""# This setup.py was autogenerated, please edit for further details
import os
from codecs import open

from setuptools import find_packages, setup

import versioneer

here = os.path.abspath(os.path.dirname(__file__))

# Dependencies.
with open('requirements.txt') as f:
    requirements = f.readlines()
install_requires = [t.strip() for t in requirements]

with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='{package_name}',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description='{package_description}',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='',
    author='{author_name}',
    author_email='{author_email}',
    maintainer='{author_name}',
    maintainer_email='{author_email}',
    python_requires='>=3',
    license='{license_detail['spdx_id']}',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords=[],
    include_package_data=True,
    packages=find_packages(),
    install_requires=install_requires,
    setup_requires=['setuptools>=38.6.0', 
                    'wheel>=0.31.0', 
                    'twine>=1.11.0']
)
"""

def setup_cfg_template(package_name):
    return f"""[metadata]
# This includes the license file in the wheel.
license_file = LICENSE

[bdist_wheel]
# This flag says to generate wheels that support both Python 2 and Python
# 3. If your code will not run unchanged on both Python 2 and 3, you will
# need to generate separate wheels for each Python version that you
# support. Removing this line (or setting universal to 0) will prevent
# bdist_wheel from trying to make a universal wheel. For more see:
# https://packaging.python.org/tutorials/distributing-packages/#wheels
universal=0

[versioneer]
VCS = git
style = pep440
versionfile_source = {package_name}/_version.py
versionfile_build = {package_name}/_version.py
tag_prefix = v
parentdir_prefix = {package_name}-
"""

def manifest(additional_text):
    return f"""include README.md
include *.txt

{additional_text}
"""

def authors_md(author_name, author_email):
    return f"""# Credits

## Maintainer

* {author_name} <{author_email}>

## Contributors

None yet. Why not be the first? See: CONTRIBUTING.md
    
"""

def contributing_md(package_name, github_username):
    return f"""# Contributing

Contributions are welcome, and they are greatly appreciated! Every
little bit helps, and credit will always be given.

## Report Bugs

Report bugs at https://github.com/{github_username}/{package_name}/issues.

If you are reporting a bug, please include:

* Any details about your local setup that might be helpful in troubleshooting.
* Detailed steps to reproduce the bug.

## Submit Feedback

The best way to send feedback is to file an issue at https://github.com/{github_username}/{package_name}/issues.

If you are proposing a feature:

* Explain in detail how it would work.
* Keep the scope as narrow as possible, to make it easier to implement.
* Remember that this is a volunteer-driven project, and that contributions
  are welcome :)
"""

def flake8_template(package_name):
    return f"""[flake8]
exclude = .git,__pycache__,build,dist,versioneer.py,{package_name}/_version.py,docs/source/conf.py
"""

def tests_example():
    return """# -*- coding: utf-8 -*-

def test_hello_world():
    text = 'Hello World'
    assert text == 'Hello World'
"""

def travis_template(package_name):
    return f"""language: python

sudo: false

dist: trusty

matrix:
  fast_finish: true
  include:
  - python: 3.6
    env: TEST_TARGET=default
  - python: 3.6
    env: TEST_TARGET=coding_standards
  allow_failures:
  - python: 3.6
    env: TEST_TARGET=coding_standards

before_install:
  - |
    URL="http://bit.ly/miniconda"
    echo ""
    if [ ! -f $HOME/miniconda/bin/conda ] ; then
      echo "Fresh miniconda installation."
      wget $URL -O miniconda.sh
      rm -rf $HOME/miniconda
      bash miniconda.sh -b -p $HOME/miniconda
    fi
  - export PATH="$HOME/miniconda/bin:$PATH"
  - conda update conda --yes
  - conda config --set show_channel_urls true
  - conda config --add channels conda-forge --force
  - conda create --yes -n TEST python=$TRAVIS_PYTHON_VERSION --file requirements.txt --file requirements-dev.txt
  - source activate TEST

# Test source distribution.
install:
  - python setup.py sdist && version=$(python setup.py --version) && pushd dist  && pip install {package_name}-${{version}}.tar.gz && popd

script:
  - if [[ $TEST_TARGET == 'default' ]]; then
      cp -r tests /tmp && cd /tmp ;
      pytest -vv tests ;
    fi
    
  - if [[ $TEST_TARGET == 'coding_standards' ]]; then
      flake8 --max-line-length=105 {package_name} ;
    fi
"""




# This setup.py was autogenerated, please edit for further details
import os
from codecs import open

from setuptools import find_packages, setup

import versioneer

here = os.path.abspath(os.path.dirname(__file__))

# Dependencies.
with open('requirements.txt') as f:
    requirements = f.readlines()
install_requires = [t.strip() for t in requirements]

with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='rppc',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description='Reproducible Python Package Creator',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='',
    author='Landung Setiawan',
    author_email='landungs@uw.edu',
    maintainer='Landung Setiawan',
    maintainer_email='landungs@uw.edu',
    python_requires='>=3',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords=[],
    include_package_data=True,
    packages=find_packages(),
    install_requires=install_requires,
    entry_points=dict(console_scripts=[
        'rppc = rppc.cli:main'
        ],
    setup_requires=['setuptools>=38.6.0', 
                    'wheel>=0.31.0', 
                    'twine>=1.11.0']
),
)
